@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright © 2013-2021 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2022 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -101,6 +101,8 @@ typedef enum {
   ROW_UP,
   ROW_DOWN,
   ROW_TAB,
+  ELEMENT_NEXT,
+  ELEMENT_PREV,
   PAGE_PREV,
   PAGE_NEXT,
   ROW_FIRST,
