@@ -3,7 +3,7 @@
  *
  * MIT/X11 License
  * Copyright © 2012 Sean Pringle <sean.pringle@gmail.com>
- * Copyright © 2013-2021 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2022 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -40,9 +40,6 @@
 
 #include "theme.h"
 
-/** The space reserved for the DOT when enabling multi-select. */
-#define DOT_OFFSET 15
-
 static void textbox_draw(widget *, cairo_t *);
 static void textbox_free(widget *);
 static int textbox_get_width(widget *);
@@ -78,7 +75,6 @@ static void textbox_resize(widget *wid, short w, short h) {
 }
 static int textbox_get_desired_height(widget *wid, const int width) {
   textbox *tb = (textbox *)wid;
-  unsigned int offset = ((tb->flags & TB_INDICATOR) ? DOT_OFFSET : 0);
   if ((tb->flags & TB_AUTOHEIGHT) == 0) {
     return tb->widget.h;
   }
@@ -88,8 +84,7 @@ static int textbox_get_desired_height(widget *wid, const int width) {
   int old_width = pango_layout_get_width(tb->layout);
   pango_layout_set_width(
       tb->layout,
-      PANGO_SCALE *
-          (width - widget_padding_get_padding_width(WIDGET(tb)) - offset));
+      PANGO_SCALE * (width - widget_padding_get_padding_width(WIDGET(tb))));
 
   int height =
       textbox_get_estimated_height(tb, pango_layout_get_line_count(tb->layout));
@@ -237,10 +232,6 @@ textbox *textbox_create(widget *parent, WidgetType type, const char *name,
   textbox_text(tb, txt ? txt : "");
   textbox_cursor_end(tb);
 
-  // auto height/width modes get handled here
-  textbox_moveresize(tb, tb->widget.x, tb->widget.y, tb->widget.w,
-                     tb->widget.h);
-
   tb->blink_timeout = 0;
   tb->blink = 1;
   if ((tb->flags & TB_EDITABLE) == TB_EDITABLE) {
@@ -255,6 +246,19 @@ textbox *textbox_create(widget *parent, WidgetType type, const char *name,
   tb->xalign = rofi_theme_get_double(WIDGET(tb), "horizontal-align", xalign);
   tb->xalign = MAX(0, MIN(1.0, tb->xalign));
 
+  if (tb->xalign < 0.2) {
+    pango_layout_set_alignment(tb->layout, PANGO_ALIGN_LEFT);
+  } else if (tb->xalign < 0.8) {
+    pango_layout_set_alignment(tb->layout, PANGO_ALIGN_CENTER);
+  } else {
+    pango_layout_set_alignment(tb->layout, PANGO_ALIGN_RIGHT);
+  }
+  // auto height/width modes get handled here
+  // UPDATE: don't autoheight here, as there is no width set.
+  // so no height can be determined and might result into  crash.
+  // textbox_moveresize(tb, tb->widget.x, tb->widget.y, tb->widget.w,
+  //                   tb->widget.h);
+
   return tb;
 }
 
@@ -321,6 +325,15 @@ static void __textbox_update_pango_text(textbox *tb) {
   } else {
     pango_layout_set_text(tb->layout, tb->text, -1);
   }
+  if (tb->text) {
+    RofiHighlightColorStyle th = {0, {0.0, 0.0, 0.0, 0.0}};
+    th = rofi_theme_get_highlight(WIDGET(tb), "text-transform", th);
+    if (th.style != 0) {
+      PangoAttrList *list = pango_attr_list_new();
+      helper_token_match_set_pango_attr_on_style(list, 0, G_MAXUINT, th);
+      pango_layout_set_attributes(tb->layout, list);
+    }
+  }
 }
 const char *textbox_get_visible_text(const textbox *tb) {
   if (tb == NULL) {
@@ -378,11 +391,10 @@ void textbox_text(textbox *tb, const char *text) {
 
 // within the parent handled auto width/height modes
 void textbox_moveresize(textbox *tb, int x, int y, int w, int h) {
-  unsigned int offset = ((tb->flags & TB_INDICATOR) ? DOT_OFFSET : 0);
   if (tb->flags & TB_AUTOWIDTH) {
     pango_layout_set_width(tb->layout, -1);
     w = textbox_get_font_width(tb) +
-        widget_padding_get_padding_width(WIDGET(tb)) + offset;
+        widget_padding_get_padding_width(WIDGET(tb));
   } else {
     // set ellipsize
     if ((tb->flags & TB_EDITABLE) == TB_EDITABLE) {
@@ -396,11 +408,9 @@ void textbox_moveresize(textbox *tb, int x, int y, int w, int h) {
 
   if (tb->flags & TB_AUTOHEIGHT) {
     // Width determines height!
-    int tw = MAX(1, w);
-    pango_layout_set_width(
-        tb->layout,
-        PANGO_SCALE *
-            (tw - widget_padding_get_padding_width(WIDGET(tb)) - offset));
+    int padding = widget_padding_get_padding_width(WIDGET(tb));
+    int tw = MAX(1 + padding, w);
+    pango_layout_set_width(tb->layout, PANGO_SCALE * (tw - padding));
     int hd = textbox_get_height(tb);
     h = MAX(hd, h);
   }
@@ -415,9 +425,8 @@ void textbox_moveresize(textbox *tb, int x, int y, int w, int h) {
 
   // We always want to update this
   pango_layout_set_width(
-      tb->layout,
-      PANGO_SCALE * (tb->widget.w -
-                     widget_padding_get_padding_width(WIDGET(tb)) - offset));
+      tb->layout, PANGO_SCALE * (tb->widget.w -
+                                 widget_padding_get_padding_width(WIDGET(tb))));
   widget_queue_redraw(WIDGET(tb));
 }
 
@@ -445,14 +454,14 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
     return;
   }
   textbox *tb = (textbox *)wid;
-  unsigned int dot_offset = ((tb->flags & TB_INDICATOR) ? DOT_OFFSET : 0);
+  int dot_offset = 0;
 
   if (tb->changed) {
     __textbox_update_pango_text(tb);
   }
 
   // Skip the side MARGIN on the X axis.
-  int x = widget_padding_get_left(WIDGET(tb));
+  int x;
   int top = widget_padding_get_top(WIDGET(tb));
   int y = (pango_font_metrics_get_ascent(tb->tbfc->metrics) -
            pango_layout_get_baseline(tb->layout)) /
@@ -467,14 +476,6 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
   }
   y += top;
 
-  x += dot_offset;
-
-  if (tb->xalign > 0.001) {
-    int rem =
-        MAX(0, tb->widget.w - widget_padding_get_padding_width(WIDGET(tb)) -
-                   line_width);
-    x = tb->xalign * rem + widget_padding_get_left(WIDGET(tb));
-  }
   // TODO check if this is still needed after flatning.
   cairo_set_operator(draw, CAIRO_OPERATOR_OVER);
   cairo_set_source_rgb(draw, 0.0, 0.0, 0.0);
@@ -485,7 +486,33 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
   }
   // Set ARGB
   // We need to set over, otherwise subpixel hinting wont work.
-  cairo_move_to(draw, x, top);
+  switch (pango_layout_get_alignment(tb->layout)) {
+  case PANGO_ALIGN_CENTER: {
+    int rem =
+        MAX(0, tb->widget.w - widget_padding_get_padding_width(WIDGET(tb)) -
+                   line_width - dot_offset);
+    x = (tb->xalign - 0.5) * rem + widget_padding_get_left(WIDGET(tb));
+    cairo_move_to(draw, x, top);
+    break;
+  }
+  case PANGO_ALIGN_RIGHT: {
+    int rem =
+        MAX(0, tb->widget.w - widget_padding_get_padding_width(WIDGET(tb)) -
+                   line_width - dot_offset);
+    x = -(1.0 - tb->xalign) * rem + widget_padding_get_left(WIDGET(tb));
+    cairo_move_to(draw, x, top);
+    break;
+  }
+  default: {
+    int rem =
+        MAX(0, tb->widget.w - widget_padding_get_padding_width(WIDGET(tb)) -
+                   line_width - dot_offset);
+    x = tb->xalign * rem + widget_padding_get_left(WIDGET(tb));
+    x += dot_offset;
+    cairo_move_to(draw, x, top);
+    break;
+  }
+  }
   cairo_save(draw);
   cairo_reset_clip(draw);
   pango_cairo_show_layout(draw, tb->layout);
@@ -510,11 +537,6 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
                     cursor_height);
     cairo_fill(draw);
   }
-
-  if ((tb->flags & TB_INDICATOR) == TB_INDICATOR && (tb->tbft & (SELECTED))) {
-    cairo_arc(draw, DOT_OFFSET / 2.0, tb->widget.h / 2.0, 2.0, 0, 2.0 * M_PI);
-    cairo_fill(draw);
-  }
 }
 
 // cursor handling for edit mode
@@ -728,9 +750,7 @@ static void textbox_cursor_del_eol(textbox *tb) {
 static void textbox_cursor_del_sol(textbox *tb) {
   if (tb && tb->cursor >= 0) {
     int length = tb->cursor;
-    if (length >= 0) {
-      textbox_delete(tb, 0, length);
-    }
+    textbox_delete(tb, 0, length);
   }
 }
 static void textbox_cursor_del_word(textbox *tb) {
@@ -878,9 +898,7 @@ void textbox_cleanup(void) {
 int textbox_get_width(widget *wid) {
   textbox *tb = (textbox *)wid;
   if (tb->flags & TB_AUTOWIDTH) {
-    unsigned int offset = (tb->flags & TB_INDICATOR) ? DOT_OFFSET : 0;
-    return textbox_get_font_width(tb) + widget_padding_get_padding_width(wid) +
-           offset;
+    return textbox_get_font_width(tb) + widget_padding_get_padding_width(wid);
   }
   return tb->widget.w;
 }
@@ -942,10 +960,8 @@ int textbox_get_desired_width(widget *wid, G_GNUC_UNUSED const int height) {
     return 0;
   }
   textbox *tb = (textbox *)wid;
-  unsigned int offset = ((tb->flags & TB_INDICATOR) ? DOT_OFFSET : 0);
   if (wid->expand && tb->flags & TB_AUTOWIDTH) {
-    return textbox_get_font_width(tb) + widget_padding_get_padding_width(wid) +
-           offset;
+    return textbox_get_font_width(tb) + widget_padding_get_padding_width(wid);
   }
   RofiDistance w = rofi_theme_get_distance(WIDGET(tb), "width", 0);
   int wi = distance_get_pixel(w, ROFI_ORIENTATION_HORIZONTAL);
@@ -959,7 +975,7 @@ int textbox_get_desired_width(widget *wid, G_GNUC_UNUSED const int height) {
   int width = textbox_get_font_width(tb);
   // Restore.
   pango_layout_set_width(tb->layout, old_width);
-  return width + padding + offset;
+  return width + padding;
 }
 
 void textbox_set_ellipsize(textbox *tb, PangoEllipsizeMode mode) {
