@@ -1,5 +1,5 @@
 /*
- * libnkutils/token - Miscellaneous utilities, token module
+ * libnkutils/xdg-theme - Miscellaneous utilities, xdg-theme module
  *
  * Copyright © 2011-2017 Quentin "Sardem FF7" Glidic
  *
@@ -31,6 +31,8 @@
 #include <locale.h>
 
 #include <glib.h>
+#include <glib-object.h>
+#include <gio/gio.h>
 
 #include <nkutils-xdg-theme.h>
 
@@ -58,7 +60,7 @@ typedef struct {
 static const struct {
     const gchar *testpath;
     NkXdgThemeTestData data;
-} _nk_uuid_tests_list[] = {
+} _nk_xdg_theme_tests_list[] = {
     {
         .testpath = "/nkutils/xdg-theme/icon/Adwaita/symbolic/found",
         .data = {
@@ -68,8 +70,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/Adwaita/scalable/actions/zoom-in-symbolic.svg",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/scalable/actions/zoom-in-symbolic.svg",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -77,12 +79,12 @@ static const struct {
         .data = {
             .type = TYPE_ICON,
             .themes = { [0] = "Adwaita" },
-            .name = "zoom-in-symbolic",
+            .name = "application-certificate-symbolic",
             .size = 48,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/Adwaita/48x48/actions/zoom-in-symbolic.symbolic.png",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/48x48/mimetypes/application-certificate-symbolic.symbolic.png",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -94,8 +96,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/Adwaita/48x48/status/trophy-gold.png",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/48x48/status/trophy-gold.png",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -107,8 +109,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/Adwaita/48x48/status/trophy-gold.png",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/48x48/status/trophy-gold.png",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -120,8 +122,8 @@ static const struct {
             .size = 16,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/hicolor/16x16/apps/geany.png",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/hicolor/16x16/apps/geany.png",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -134,8 +136,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/Adwaita/scalable/status/network-wireless-signal-ok-symbolic.svg",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/scalable/status/network-wireless-signal-ok-symbolic.svg",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -149,7 +151,7 @@ static const struct {
             .scale = 1,
             .svg = TRUE,
             .result = NULL,
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -162,8 +164,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/Adwaita/scalable/emblems/emblem-favorite-symbolic.svg",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/scalable/emblems/emblem-favorite-symbolic.svg",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -175,8 +177,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/gnome/scalable/actions/zoom-in-symbolic.svg",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/scalable/actions/zoom-in-symbolic.svg",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -188,8 +190,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/gnome/48x48/actions/zoom-in.png",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/48x48/actions/zoom-in.png",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -201,8 +203,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/gnome/48x48/status/trophy-gold.png",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/48x48/status/trophy-gold.png",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -214,8 +216,8 @@ static const struct {
             .size = 16,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/hicolor/16x16/apps/geany.png",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/hicolor/16x16/apps/geany.png",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -228,8 +230,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/gnome/scalable/status/network-wireless-signal-ok-symbolic.svg",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/scalable/status/network-wireless-signal-ok-symbolic.svg",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -243,7 +245,7 @@ static const struct {
             .scale = 1,
             .svg = TRUE,
             .result = NULL,
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -256,8 +258,8 @@ static const struct {
             .size = 48,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/gnome/scalable/emblems/emblem-favorite-symbolic.svg",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/scalable/emblems/emblem-favorite-symbolic.svg",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -268,8 +270,8 @@ static const struct {
             .size = 18,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/hicolor/16x16/apps/geany.png",
-            .theme_test = "/usr/share/icons/hicolor/index.theme",
+            .result = "icons/hicolor/16x16/apps/geany.png",
+            .theme_test = "icons/hicolor/index.theme",
         }
     },
     {
@@ -280,8 +282,8 @@ static const struct {
             .size = 18,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/hicolor/16x16/apps/pidgin.png",
-            .theme_test = "/usr/share/icons/hicolor/index.theme",
+            .result = "icons/hicolor/16x16/apps/pidgin.png",
+            .theme_test = "icons/hicolor/index.theme",
         }
     },
     {
@@ -293,8 +295,8 @@ static const struct {
             .size = 0,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/hicolor/scalable/apps/pidgin.svg",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/hicolor/scalable/apps/pidgin.svg",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -305,7 +307,7 @@ static const struct {
             .size = 19,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/pixmaps/htop.png",
+            .result = "pixmaps/htop.png",
         }
     },
     {
@@ -316,7 +318,18 @@ static const struct {
             .size = 0,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/pixmaps/debian-logo.png",
+            .result = "pixmaps/debian-logo.png",
+        }
+    },
+    {
+        .testpath = "/nkutils/xdg-theme/icon/no-theme/size/fallback/home-icons",
+        .data = {
+            .type = TYPE_ICON,
+            .name = "test-icon-home-icons",
+            .size = 0,
+            .scale = 1,
+            .svg = FALSE,
+            .result = SRCDIR "/tests/home/.icons/test-icon-home-icons.png",
         }
     },
     {
@@ -324,12 +337,12 @@ static const struct {
         .data = {
             .type = TYPE_ICON,
             .themes = { [0] = "Adwaita" },
-            .name = "edit-find-symbolic",
+            .name = "application-certificate-symbolic",
             .size = 0,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/Adwaita/96x96/actions/edit-find-symbolic.symbolic.png",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/96x96/mimetypes/application-certificate-symbolic.symbolic.png",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -341,8 +354,8 @@ static const struct {
             .size = 0,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/Adwaita/scalable/actions/edit-find-symbolic.svg",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/scalable/actions/edit-find-symbolic.svg",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -354,8 +367,8 @@ static const struct {
             .size = 19,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/Adwaita/22x22/actions/edit-find.png",
-            .theme_test = "/usr/share/icons/Adwaita/index.theme",
+            .result = "icons/Adwaita/22x22/actions/edit-find.png",
+            .theme_test = "icons/Adwaita/index.theme",
         }
     },
     {
@@ -367,8 +380,8 @@ static const struct {
             .size = 0,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/gnome/96x96/actions/edit-find-symbolic.symbolic.png",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/96x96/actions/edit-find-symbolic.symbolic.png",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -380,8 +393,8 @@ static const struct {
             .size = 0,
             .scale = 1,
             .svg = TRUE,
-            .result = "/usr/share/icons/gnome/scalable/actions/edit-find-symbolic.svg",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/scalable/actions/edit-find-symbolic.svg",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -393,8 +406,8 @@ static const struct {
             .size = 19,
             .scale = 1,
             .svg = FALSE,
-            .result = "/usr/share/icons/gnome/22x22/actions/edit-find.png",
-            .theme_test = "/usr/share/icons/gnome/index.theme",
+            .result = "icons/gnome/22x22/actions/edit-find.png",
+            .theme_test = "icons/gnome/index.theme",
         }
     },
     {
@@ -429,7 +442,7 @@ static const struct {
             .size = 10,
             .scale = 1,
             .svg = TRUE,
-            .result = SRCDIR G_DIR_SEPARATOR_S "tests/icons/recursive-theme-test/test-dir/test-icon.svg",
+            .result = SRCDIR "/tests/icons/recursive-theme-test/test-dir/test-icon.svg",
         }
     },
     {
@@ -439,8 +452,8 @@ static const struct {
             .themes = { [0] = "freedesktop" },
             .name = "network-connectivity-established",
             .profile = "stereo",
-            .result = "/usr/share/sounds/freedesktop/stereo/network-connectivity-established.oga",
-            .theme_test = "/usr/share/sounds/freedesktop/index.theme",
+            .result = "sounds/freedesktop/stereo/network-connectivity-established.oga",
+            .theme_test = "sounds/freedesktop/index.theme",
         }
     },
     {
@@ -450,8 +463,8 @@ static const struct {
             .themes = { [0] = "freedesktop" },
             .name = "bell-too-specific",
             .profile = "stereo",
-            .result = "/usr/share/sounds/freedesktop/stereo/bell.oga",
-            .theme_test = "/usr/share/sounds/freedesktop/index.theme",
+            .result = "sounds/freedesktop/stereo/bell.oga",
+            .theme_test = "sounds/freedesktop/index.theme",
         }
     },
     {
@@ -461,8 +474,8 @@ static const struct {
             .themes = { [0] = "non-existing-i-hope" },
             .name = "dialog-information",
             .profile = "stereo",
-            .result = "/usr/share/sounds/freedesktop/stereo/dialog-information.oga",
-            .theme_test = "/usr/share/sounds/freedesktop/index.theme",
+            .result = "sounds/freedesktop/stereo/dialog-information.oga",
+            .theme_test = "sounds/freedesktop/index.theme",
         }
     },
     {
@@ -472,35 +485,95 @@ static const struct {
             .themes = { [0] = "purple" },
             .name = "logout",
             .profile = "stereo",
-            .result = "/usr/share/sounds/purple/logout.wav",
+            .result = "sounds/purple/logout.wav",
         }
     },
 };
 
+static gchar *
+_nk_xdg_theme_file_canonicalize(gchar *path)
+{
+    if ( path == NULL )
+        return NULL;
+
+    GFile *file;
+    file = g_file_new_for_path(path);
+    g_free(path);
+
+    path = g_file_get_path(file);
+    g_object_unref(file);
+
+    return path;
+}
+
+static gchar *
+_nk_xdg_theme_file_exists(const gchar *file)
+{
+    if ( g_path_is_absolute(file) )
+    {
+        if ( ! g_file_test(file, G_FILE_TEST_IS_REGULAR) )
+            return NULL;
+        return _nk_xdg_theme_file_canonicalize(g_strdup(file));
+    }
+
+    const gchar * const *system_dirs = g_get_system_data_dirs();
+    const gchar * const *system_dir;
+
+    for ( system_dir = system_dirs ; *system_dir != NULL ; ++system_dir )
+    {
+        gchar *full_path;
+        full_path = g_build_filename(*system_dir, file, NULL);
+        if ( g_file_test(full_path, G_FILE_TEST_IS_REGULAR) )
+            return _nk_xdg_theme_file_canonicalize(full_path);
+        g_free(full_path);
+    }
+
+    return NULL;
+}
+
 static void
-_nk_uuid_tests_func(gconstpointer user_data)
+_nk_xdg_theme_tests_func(gconstpointer user_data)
 {
     const NkXdgThemeTestData *data = user_data;
 
-    if ( ( ( data->result != NULL ) && ( ! g_file_test(data->result, G_FILE_TEST_IS_REGULAR) ) )
-         || ( ( data->theme_test != NULL ) && ( ! g_file_test(data->theme_test, G_FILE_TEST_IS_REGULAR) ) ) )
+    gchar *theme_test = NULL;
+    gchar *expected = NULL;
+
+    if ( data->theme_test != NULL )
     {
-        g_test_skip("Theme not installed");
-        return;
+        theme_test = _nk_xdg_theme_file_exists(data->theme_test);
+        if ( theme_test == NULL )
+        {
+            g_test_skip("Theme not installed");
+            return;
+        }
+        g_free(theme_test);
     }
 
-    gchar *file = NULL;
+    if ( data->result != NULL )
+    {
+        expected = _nk_xdg_theme_file_exists(data->result);
+        if ( expected == NULL )
+        {
+            g_test_skip("Icon file not installed");
+            return;
+        }
+    }
+
+    gchar *found = NULL;
     switch ( data->type )
     {
     case TYPE_ICON:
-        file = nk_xdg_theme_get_icon(context, data->themes, data->context, data->name, data->size, data->scale, data->svg);
+        found = nk_xdg_theme_get_icon(context, data->themes, data->context, data->name, data->size, data->scale, data->svg);
     break;
     case TYPE_SOUND:
-        file = nk_xdg_theme_get_sound(context, data->themes, data->name, data->profile, NULL);
+        found = nk_xdg_theme_get_sound(context, data->themes, data->name, data->profile, NULL);
     break;
     }
-    g_assert_cmpstr(file, ==, data->result);
-    g_free(file);
+    found = _nk_xdg_theme_file_canonicalize(found);
+    g_assert_cmpstr(found, ==, expected);
+    g_free(found);
+    g_free(expected);
 }
 
 int
@@ -512,10 +585,9 @@ main(int argc, char *argv[])
 
     g_test_set_nonfatal_assertions();
 
-    g_setenv("HOME", "/var/empty", TRUE);
+    g_setenv("HOME", SRCDIR G_DIR_SEPARATOR_S "tests" G_DIR_SEPARATOR_S "home", TRUE);
     g_setenv("XDG_CONFIG_HOME", SRCDIR G_DIR_SEPARATOR_S "tests", TRUE);
     g_setenv("XDG_DATA_HOME", SRCDIR G_DIR_SEPARATOR_S "tests", TRUE);
-    g_setenv("XDG_DATA_DIRS", "/usr/share/", TRUE);
     g_setenv("XDG_SESSION_DESKTOP", "", TRUE);
     g_setenv("XDG_CURRENT_DESKTOP", "", TRUE);
     g_setenv("GNOME_DESKTOP_SESSION_ID", "", TRUE);
@@ -523,8 +595,8 @@ main(int argc, char *argv[])
     g_setenv("DESKTOP_SESSION", "", TRUE);
 
     gsize i;
-    for ( i = 0 ; i < G_N_ELEMENTS(_nk_uuid_tests_list) ; ++i )
-        g_test_add_data_func(_nk_uuid_tests_list[i].testpath, &_nk_uuid_tests_list[i].data, _nk_uuid_tests_func);
+    for ( i = 0 ; i < G_N_ELEMENTS(_nk_xdg_theme_tests_list) ; ++i )
+        g_test_add_data_func(_nk_xdg_theme_tests_list[i].testpath, &_nk_xdg_theme_tests_list[i].data, _nk_xdg_theme_tests_func);
 
     context = nk_xdg_theme_context_new(NULL, NULL);
     int ret = g_test_run();
